@@ -1,11 +1,11 @@
 /***************************************************************************
 						buildinglayer.cpp  -  description
 							-------------------
-	begin                : sam sep 20 2003
-	copyright            : (C) 2003-2006 by Duong-Khang NGUYEN
+	begin                : september 20th, 2003
+	copyright            : (C) 2003-2007 by Duong-Khang NGUYEN
 	email                : neoneurone @ users sourceforge net
-	
-	$Id: buildinglayer.cpp 51 2006-09-27 22:15:47Z neoneurone $
+
+	$Id: buildinglayer.cpp 261 2007-10-20 18:22:47Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -17,24 +17,23 @@
  *                                                                         *
  ***************************************************************************/
 
+// Useful enumerations
+#include "opencity_direction.h"
+#include "opencity_structure_type.h"
+
+// OpenCity header
 #include "buildinglayer.h"
-
 #include "city.h"
-#include "map.h"
-
 #include "rcistructure.h"
 #include "wegstructure.h"
 #include "pathstructure.h"
 #include "treestructure.h"
-
 #include "guicontainer.h"
 #include "guibutton.h"
 
-#include "propertymanager.h"
-
-
-extern PropertyManager* gpPropertyMgr;	// global property manager
-extern Map* gpMapMgr;					// global map height manager
+// Global settings
+#include "globalvar.h"
+extern GlobalVar gVars;
 
 
    /*=====================================================================*/
@@ -45,8 +44,7 @@ BuildingLayer::BuildingLayer
 {
 	OPENCITY_DEBUG( "ctor" );
 // initialization of width and height
-	_uiLayerWidth = rcCity.cityGetWidth();
-	_uiLayerLength = rcCity.cityGetHeight();
+	rcCity.GetWL( _uiLayerWidth, _uiLayerLength );
 
 // initialization of the table of pointer of Structure
 	uint citySurface = _uiLayerLength * _uiLayerWidth;
@@ -65,7 +63,6 @@ BuildingLayer::~BuildingLayer()
 
 	uint citySurface = _uiLayerLength * _uiLayerWidth;
 	for ( uint counter = 0; counter < citySurface; counter++ ) {
-	// delete NULL pointer is allowed ?
 	// delete seems to have no effects on NULL
 		if (_tabpStructure[counter] != NULL) {
 			delete _tabpStructure[counter];
@@ -74,6 +71,7 @@ BuildingLayer::~BuildingLayer()
 	}
 
 	delete [] _tabpStructure;
+	_tabpStructure = NULL;
 }
 
 
@@ -90,21 +88,22 @@ BuildingLayer::SaveTo( std::fstream& rfs )
 
 	for ( l = 0; l < _uiLayerLength; l++ )
 	for ( w = 0; w < _uiLayerWidth; w++ ) {
-// debug
-//		rfs << w << " " << l << std::endl;
-
 		p = _tabpStructure[linear++];
 
-	// We save only extra information for main structures
+	// IF this is a part of a bigger structure THEN
 		if ((p != NULL) && (p->GetMain() != NULL)) {
 			p = NULL;
 		}
-		rfs << p << std::ends;
 
+	// IF there is structure's information to save THEN
 		if (p != NULL) {
+			rfs << 1 << std::ends;
 			rfs << p->GetType() << std::ends;
 			p->SaveTo( rfs );
 		}
+		else {
+			rfs << 0 << std::ends;
+		}
 	} // for
 }
 
@@ -115,9 +114,9 @@ BuildingLayer::LoadFrom( std::fstream& rfs )
 {
 	OPENCITY_DEBUG( __PRETTY_FUNCTION__ << "loading" );
 	Structure* p = NULL;
-	void* t = NULL;
+	uint extra = 0;			// Extra info ? 1 = true; 0 = false;
 	uint w = 0, l = 0, anUint = 0, linear = 0;
-	OPENCITY_STRUCTURE_TYPE type = OC_TYPE_UNUSED;		///< Structure's object type
+	OPENCITY_STRUCTURE_TYPE type = OC_TYPE_UNDEFINED;		///< Structure's object type
 
 // Remove the old structures
 	uint citySurface = _uiLayerLength * _uiLayerWidth;
@@ -140,79 +139,49 @@ BuildingLayer::LoadFrom( std::fstream& rfs )
 		_tabpStructure[linear] = NULL;
 	}
 
-// debug
-//	uint wt = 0, lt = 0;
-
 // Load the structures
 	for ( l = 0; l < _uiLayerLength; l++ )
 	for ( w = 0; w < _uiLayerWidth; w++ ) {
-// debug
-//		rfs >> wt; rfs.ignore();
-//		rfs >> lt; rfs.ignore();
+		rfs >> extra; rfs.ignore();
 
-		rfs >> t; rfs.ignore();
-//		OPENCITY_DEBUG( "Current w/l: " << w << " / " << l << " | Reading w/l/t: " << wt << " / " << lt << " / " << t );
+	// IF there is no extra information to read THEN continue
+		if (extra == 0)
+			continue;
 
-		if (t != NULL) {
-			rfs >> anUint; rfs.ignore(); type = (OPENCITY_STRUCTURE_TYPE)anUint;
-			switch (type) {
-				case OC_TYPE_RESIDENCE:
-				case OC_TYPE_COMMERCE:
-				case OC_TYPE_INDUSTRY:
-					p = new RCIStructure();
-					break;
+		rfs >> anUint; rfs.ignore(); type = (OPENCITY_STRUCTURE_TYPE)anUint;
+		switch (type) {
+			case OC_TYPE_RESIDENCE:
+			case OC_TYPE_COMMERCE:
+			case OC_TYPE_INDUSTRY:
+				p = new RCIStructure();
+				break;
 
-				case OC_TYPE_WATER:
-				case OC_TYPE_ELECTRICITY:
-				case OC_TYPE_GAS:
-				case OC_TYPE_GOVERNMENT:		// hack
-					p = new WEGStructure();
-					break;
+			case OC_TYPE_WATER:
+			case OC_TYPE_ELECTRICITY:
+			case OC_TYPE_GAS:
+			case OC_TYPE_GOVERNMENT:		// hack
+				p = new WEGStructure();
+				break;
 
-				case OC_TYPE_PATH:
-					p = new PathStructure();
-					break;
+			case OC_TYPE_PATH:
+				p = new PathStructure();
+				break;
 
-				case OC_TYPE_TREE:
-					p = new TreeStructure();
-					break;
+			case OC_TYPE_TREE:
+				p = new TreeStructure();
+				break;
 
-				default:
-					OPENCITY_DEBUG( "Unknown structure's type: " << type );
-					assert( 0 );
-			}
+			default:
+				OPENCITY_DEBUG( "Unknown structure's type: " << type );
+				assert( 0 );
+		}
 
-			p->LoadFrom( rfs );
-			_LoadStructure( w, l, p );
-		} // if (p != NULL)
+		p->LoadFrom( rfs );
+		_LoadStructure( w, l, p );
 	} // for
 }
 
 
-   /*=====================================================================*/
-void
-BuildingLayer::uiKeyboard( const SDL_KeyboardEvent & rcsSDLKeyboardEvent )
-{}
-
-void
-BuildingLayer::uiMouseMotion( const SDL_MouseMotionEvent &
-				rcsSDLMouseMotionEvent )
-{}
-
-void
-BuildingLayer::uiMouseButton( const SDL_MouseButtonEvent & 
-				rcsSDLMouseButtonEvent )
-{}
-
-void
-BuildingLayer::uiExpose( const SDL_ExposeEvent & rcsSDLExposeEvent )
-{}
-
-void
-BuildingLayer::uiResize( const SDL_ResizeEvent & rcsSDLResizeEvent )
-{}
-
-
    /*=====================================================================*/
 const bool
 BuildingLayer::IsConstructive(
@@ -239,6 +208,7 @@ BuildingLayer::IsConstructive(
 			case OC_STRUCTURE_PARK:
 			case OC_STRUCTURE_FLORA:
 			case OC_STRUCTURE_EPLANT_COAL:
+			case OC_STRUCTURE_EPLANT_NUCLEAR:
 			case OC_STRUCTURE_FIREDEPT:
 			case OC_STRUCTURE_POLICEDEPT:
 			case OC_STRUCTURE_HOSPITALDEPT:
@@ -248,7 +218,7 @@ BuildingLayer::IsConstructive(
 			// OR the square is not plane
 			// then the square is not constructive
 				if ((this->GetLinearStructure(linearIndex) != NULL)
-				  ||(gpMapMgr->IsSquarePlane(w, L1) != true ))
+				  ||(gVars.gpMapMgr->IsSquarePlane(w, L1) != true ))
 					return false;
 				break;
 
@@ -292,7 +262,7 @@ BuildingLayer::BuildPreview(
 
 // FIXME: Work for WEG structure only !
 // Get the graphic code of the structure
-	enumGraphicCode = gpPropertyMgr->GetGC( enumStructCode );
+	enumGraphicCode = gVars.gpPropertyMgr->GetGC( enumStructCode );
 	if ( enumGraphicCode == OC_EMPTY ) {
 		OPENCITY_DEBUG( "WARNING: not implemented" );
 		assert(0);
@@ -300,7 +270,7 @@ BuildingLayer::BuildPreview(
 
 // Get the corresponding WLH and calculate the range
 // An coal electric plant is 4x4 size by default
-	gpPropertyMgr->GetWLH( enumGraphicCode, sw, 4, sl, 4, sh, 1 );
+	gVars.gpPropertyMgr->GetWLH( enumGraphicCode, sw, 4, sl, 4, sh, 1 );
 	W1 = W;
 	L1 = L;
 	W2 = W + sw - 1;
@@ -326,7 +296,7 @@ BuildingLayer::BuildStructure(
 	const OPENCITY_STRUCTURE_CODE & enumStructCode,
 	uint& rCost )
 {
-	OPENCITY_DEBUG( "I'm building some structures" );
+//	OPENCITY_DEBUG( "I'm building some structures" );
 
 // For more obvious coding
 	OPENCITY_ERR_CODE errCode;
@@ -345,6 +315,7 @@ BuildingLayer::BuildStructure(
 		case OC_STRUCTURE_POLICEDEPT:
 		case OC_STRUCTURE_HOSPITALDEPT:
 		case OC_STRUCTURE_EPLANT_COAL:
+		case OC_STRUCTURE_EPLANT_NUCLEAR:
 			errCode = _BuildWEGStructure( W1, L1, enumStructCode, rCost );
 			break;
 
@@ -368,6 +339,46 @@ BuildingLayer::BuildStructure(
 }
 
 
+   /*=====================================================================*/
+const bool
+BuildingLayer::GetMainWL
+(
+	const Structure* const pstruct,
+	uint & w,
+	uint & l
+) const
+{
+	uint sw = 0, sl = 0, sh = 0;
+	uint w1 = w, l1 = l, w2 = w, l2 = l;
+	uint linearIndex = 0;
+	const Structure* pMain = NULL;
+
+	assert( pstruct != NULL );
+	pMain = (pstruct->GetMain() != NULL) ? pstruct->GetMain() : pstruct;
+
+// Calculate the possible area to limit cpu usage
+	gVars.gpPropertyMgr->GetWLH( pMain->GetGraphicCode(), sw, 0, sl, 0, sh, 0 );
+	assert( (sw != 0) && (sl != 0) );
+	sw--; sl--;			// We calculate the range
+
+	gVars.gpMapMgr->GetPossibleWH( w1, l1, -sw, -sl );
+	gVars.gpMapMgr->GetPossibleWH( w2, l2,  sw,  sl );
+
+// Now search for all STRUCTURE_PART and the main structure, destroy em
+	for ( l = l1; l <= l2; l++ ) {
+		linearIndex = l*_uiLayerWidth + w1;
+		for ( w = w1; w <= w2; w++ ) {
+		// WARNING: pointer comparaison !
+			if ( pMain == _tabpStructure[ linearIndex ] )
+				return true;
+			linearIndex++;
+		}
+	}
+
+	return false;
+}
+
+
    /*=====================================================================*/
 const OPENCITY_ERR_CODE
 BuildingLayer::ResizeStructure(
@@ -380,13 +391,13 @@ BuildingLayer::ResizeStructure(
 	uint ow = 0, ol = 0, oh = 0;					// Old structure dimensions
 	uint nw = 0, nl = 0, nh = 0;					// New structure dimensions
 	uint xw = 0, xl = 0, xh = 0;					// Destroyed structure dimensions
+	uint sxw = 0, sxl = 0;							// Destroyed structure coordinates
 	uint dw, dl;									// Used in the "for loop"
 	uint uiCost;									// Temp cost, necessary to call internal methods
 
 	OPENCITY_STRUCTURE_CODE structCode;
 	OPENCITY_STRUCTURE_CODE destroyedSC;			// The destroyed structure's code. It's used for rebuild
 
-
 // Get the pointer to the structure that we need to modify
 	pStruct = _tabpStructure[ linearIndex ];
 	assert( pStruct != NULL );
@@ -394,10 +405,12 @@ BuildingLayer::ResizeStructure(
 // IF the graphic code has not changed THEN it's okay
 	if ( oldGC == pStruct->GetGraphicCode() )
 		return OC_ERR_FREE;
+
+//	OPENCITY_DEBUG( "Begin - W/L " << w << "/" << l );
 	structCode = pStruct->GetCode();
 
 // Get the old WLH
-	gpPropertyMgr->GetWLH(
+	gVars.gpPropertyMgr->GetWLH(
 		oldGC,
 		ow, 0,
 		ol, 0,
@@ -405,7 +418,7 @@ BuildingLayer::ResizeStructure(
 	assert((ow != 0) && (ol != 0));		// not used yet: && (oh != 0) 
 
 // Get the new WLH
-	gpPropertyMgr->GetWLH(
+	gVars.gpPropertyMgr->GetWLH(
 		pStruct->GetGraphicCode(),
 		nw, 0,
 		nl, 0,
@@ -440,21 +453,27 @@ BuildingLayer::ResizeStructure(
 		// AND it's not the current structure's part neither THEN remove it
 			else if (pTemp != pStruct and pTemp->GetMain() != pStruct) {
 			// Remember the structure which is going to be destroyed
+				sxw = dw; sxl = dl;
+				if (GetMainWL(pTemp, sxw, sxl) == false) {
+					assert(0);
+				}
 				if (pTemp->GetCode() == OC_STRUCTURE_PART)
 					pMain = pTemp->GetMain();
 				else
 					pMain = pTemp;
 
+			// Save the structure code and its main coordinates
 				destroyedSC = pMain->GetCode();
-				gpPropertyMgr->GetWLH( pMain->GetGraphicCode(), xw, 0, xl, 0, xh, 0 );
+				gVars.gpPropertyMgr->GetWLH( pMain->GetGraphicCode(), xw, 0, xl, 0, xh, 0 );
 
+//				OPENCITY_DEBUG( "Delete structure - W/L " << dw << "/" << dl );
 				_DestroyStructure( dw, dl, uiCost );
 				pTemp = new RCIStructure( OC_STRUCTURE_PART, pStruct );
 				_tabpStructure[ linearIndex ] = pTemp;
 
 				// Rebuild the neighbourhood
 				if (xw > 1 or xl > 1) {
-					BuildStructure( dw, dl, dw + xw-1, dl + xl-1, destroyedSC, uiCost );
+					BuildStructure( sxw, sxl, dw + xw-1, dl + xl-1, destroyedSC, uiCost );
 				}
 			}
 			pTemp->Set( OC_STRUCTURE_MARK );
@@ -467,17 +486,23 @@ BuildingLayer::ResizeStructure(
 	for (dl = l; dl < l + ol; dl++) {
 		linearIndex = (dl*_uiLayerWidth) + w;
 		for (dw = w; dw < w + ow; dw++) {
+//			OPENCITY_DEBUG( "Delete unmarked structure - W/L " << dw << "/" << dl );
 			pTemp = _tabpStructure[ linearIndex ];
 // debug
 			assert( pTemp != NULL );
 			if (pTemp->IsSet(OC_STRUCTURE_MARK) == false) {
-				_DestroyStructure( dw, dl, uiCost );
+				// We don't use this _DestroyStructure( dw, dl, uiCost ) because it will
+				// get the structure's size and delete the whole structure
+				// that we're working on
+				delete _tabpStructure[ linearIndex ];
 				_tabpStructure[ linearIndex ] = new RCIStructure( structCode );
 			}
 			linearIndex++;
 		} // for dw
 	} // for dl
 
+//	OPENCITY_DEBUG( "End - W/L " << w << "/" << l );
+
 	return OC_ERR_FREE;
 }
 
@@ -502,7 +527,10 @@ BuildingLayer::DestroyStructure(
 			rCost += costPerSquare;
 	}
 
-	return OC_ERR_FREE;
+	if (rCost > 0)
+		return OC_ERR_FREE;
+	else
+		return OC_ERR_SOMETHING;
 }
 
 
@@ -534,12 +562,12 @@ BuildingLayer::GetRandomStructure(
 	if (enumStructCode == OC_STRUCTURE_UNDEFINED)
 		return NULL;
 
-// nothing can be static because we can be called by more than 1 thread
+// Nothing can be static because we can be called by more than 1 thread
 	uint linear;
 	uint counter;
 	Structure* pstruct;
 
-// we try 100 times before giving up
+// We try 100 times before giving up
 // OC_CHANCE_COUNTER_MAX = 100
 	counter = 0;
 	do {
@@ -556,7 +584,6 @@ BuildingLayer::GetRandomStructure(
 		counter++;
 	} while (counter < OC_CHANCE_COUNTER_MAX);
 
-//debug	cout << "coco: " << linear << endl;
 	return NULL;
 }
 
@@ -624,6 +651,7 @@ BuildingLayer::ContainStructure(
 		case OC_STRUCTURE_ROAD:
 		case OC_STRUCTURE_ELINE:
 		case OC_STRUCTURE_EPLANT_COAL:
+		case OC_STRUCTURE_EPLANT_NUCLEAR:
 		case OC_STRUCTURE_FIREDEPT:
 		case OC_STRUCTURE_POLICEDEPT:
 		case OC_STRUCTURE_EDUCATIONDEPT:
@@ -683,7 +711,7 @@ BuildingLayer::ContainStructureOnly(
 
 
 	uint w = 0, l = 0, linear = 0;
-	Structure* pstruct = NULL;
+	Structure* pstruct = NULL, *pMain = NULL;
 
 	switch (enumStructCode) {
 		case OC_STRUCTURE_UNDEFINED:
@@ -700,6 +728,7 @@ BuildingLayer::ContainStructureOnly(
 		case OC_STRUCTURE_ROAD:
 		case OC_STRUCTURE_ELINE:
 		case OC_STRUCTURE_EPLANT_COAL:
+		case OC_STRUCTURE_EPLANT_NUCLEAR:
 		case OC_STRUCTURE_FIREDEPT:
 		case OC_STRUCTURE_POLICEDEPT:
 		case OC_STRUCTURE_EDUCATIONDEPT:
@@ -707,9 +736,14 @@ BuildingLayer::ContainStructureOnly(
 			for (l = L1; l <= L2; l++) {
 				linear = l*_uiLayerWidth + W1;
 				for (w = W1; w <= W2; w++, linear++) {
+				// WARNING: IF this is a "part" structure
+				// THEN we look for the main structure code
 					pstruct = GetLinearStructure( linear );
 					if (pstruct != NULL) {
-						if (pstruct->GetCode() != enumStructCode)
+						pMain = pstruct->GetMain();
+						if (pMain == NULL)
+							pMain = pstruct;
+						if (pMain->GetCode() != enumStructCode)
 							return false;
 					}
 				}  // while w
@@ -750,8 +784,8 @@ BuildingLayer::QueryStructure(
 {
 	GUIContainer* pcontainer = new GUIContainer( 100, 100, 140, 140 );
 	Structure* pstruct = _tabpStructure[ l*_uiLayerWidth + w ];
-	OPENCITY_COLOR red = { 255, 0, 0, 255 };
-	OPENCITY_COLOR green = { 0, 255, 0, 255 };
+	Color red = { 255, 0, 0, 255 };
+	Color green = { 0, 255, 0, 255 };
 
 // everything is RED :D
 	Layer::pbtnQW->SetBackground( red );
@@ -789,6 +823,26 @@ BuildingLayer::QueryStructure(
 }
 
 
+   /*=====================================================================*/
+void
+BuildingLayer::Keyboard( const SDL_KeyboardEvent& rcEvent )
+{}
+
+void
+BuildingLayer::MouseMotion( const SDL_MouseMotionEvent& rcEvent )
+{}
+
+void
+BuildingLayer::MouseButton( const SDL_MouseButtonEvent& rcEvent )
+{}
+
+void
+BuildingLayer::Expose( const SDL_ExposeEvent& rcEvent )
+{}
+
+void
+BuildingLayer::Resize( const SDL_ResizeEvent& rcEvent )
+{}
 
 
    /*=====================================================================*/
@@ -815,7 +869,7 @@ BuildingLayer::_IsPathConstructive(
 		return false;
 
 // Calculate the min and the max heights
-	gpMapMgr->GetSquareHeight( w, h, tabH );
+	gVars.gpMapMgr->GetSquareHeight( w, h, tabH );
 	for (uiIndex = 0; uiIndex < 4; uiIndex++) {
 		if (tabH[uiIndex] < minH)
 			minH = tabH[uiIndex];
@@ -855,13 +909,12 @@ BuildingLayer::_BuildPathStructure(
 {
 	OPENCITY_DEBUG( "I'm building some path structures" );
 
-	uint w, h;
-	uint cost;
+	uint w = 0, l = 0;
+	uint cost = 0;
 
 	OPENCITY_SWAP( W1, W2, uint );
 	OPENCITY_SWAP( L1, L2, uint );
 
-	cost = 0;
 	rCost = 0;
 
 // is the area road constructive ?
@@ -872,14 +925,14 @@ BuildingLayer::_BuildPathStructure(
 			if (_IsPathConstructive( w, L1, enumStructCode ) == false)
 				return OC_ERR_SOMETHING;
 	// we test the rest
-		for (h = L1; h <= L2; h++)
-			if (_IsPathConstructive( W2, h, enumStructCode ) == false)
+		for (l = L1; l <= L2; l++)
+			if (_IsPathConstructive( W2, l, enumStructCode ) == false)
 				return OC_ERR_SOMETHING;
 	}
 	else {
 	// we test the largest side first
-		for (h = L1; h < L2; h++)
-			if (_IsPathConstructive( W1, h, enumStructCode ) == false)
+		for (l = L1; l < L2; l++)
+			if (_IsPathConstructive( W1, l, enumStructCode ) == false)
 				return OC_ERR_SOMETHING;
 	// we test the rest
 		for (w = W1; w <= W2; w++)
@@ -889,7 +942,7 @@ BuildingLayer::_BuildPathStructure(
 
 
 // it's OK, let's GO !
-	uint linearIndex;
+	uint linearIndex = 0;
 
 	if (W2 - W1 >= L2 - L1) {
 	// we build the largest side first
@@ -902,16 +955,16 @@ BuildingLayer::_BuildPathStructure(
 	// then we build the rest
 // this is already calculated by the previous "for"
 //		linearIndex = (L1*_uiLayerWidth) + W2;
-		for (h = L1; h <= L2; h++, linearIndex += _uiLayerWidth) {
-			_BuildPathStructure( W2, h, linearIndex, enumStructCode, cost );
+		for (l = L1; l <= L2; l++, linearIndex += _uiLayerWidth) {
+			_BuildPathStructure( W2, l, linearIndex, enumStructCode, cost );
 			rCost += cost;
 		}
 	}
 	else {
 	// we build the largest side first
 		linearIndex = (L1*_uiLayerWidth) + W1;
-		for (h = L1; h < L2; h++, linearIndex += _uiLayerWidth) {
-			_BuildPathStructure( W1, h, linearIndex, enumStructCode, cost );
+		for (l = L1; l < L2; l++, linearIndex += _uiLayerWidth) {
+			_BuildPathStructure( W1, l, linearIndex, enumStructCode, cost );
 			rCost += cost;
 		}
 
@@ -932,15 +985,15 @@ BuildingLayer::_BuildPathStructure(
 void
 BuildingLayer::_BuildPathStructure(
 	const uint & w,
-	const uint & h,
+	const uint & l,
 	const uint & linearIndex,
 	const OPENCITY_STRUCTURE_CODE & enumStructCode,
 	uint& rCost )
 {
-	Structure* pstructNeighbour = NULL;
-	PathStructure* ppathstructNeighbour;
-	PathStructure* pNewStructure;
-	uint nW, nH;
+	Structure* pstructNeighbour				= NULL;
+	PathStructure* ppathstructNeighbour		= NULL;
+	PathStructure* pNewStructure			= NULL;
+	uint nW = 0, nL = 0;
 
 
 // IF there's already something, THEN we return
@@ -951,12 +1004,12 @@ BuildingLayer::_BuildPathStructure(
 
 // Build the new road
 	pNewStructure = new PathStructure(enumStructCode);
-	rCost = gpPropertyMgr->Get( OC_BUILD_COST, enumStructCode );
+	rCost = gVars.gpPropertyMgr->Get( OC_TOOL_COST, enumStructCode );
 	_tabpStructure[ linearIndex ] = pNewStructure;
 
 // Get the neighbour in the North
-	if (gpMapMgr->GetNeighbourWH( w, h, nW, nH, OC_DIR_N ) == true) {
-		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nL, OC_DIR_O_N ) == true) {
+		pstructNeighbour = BuildingLayer::GetStructure( nW, nL );
 	// if the neighbour is a road structure
 	// then add this new road structure as its new neighbour
 	// we do the same with the new road structure
@@ -965,41 +1018,41 @@ BuildingLayer::_BuildPathStructure(
 //debug cout << "found 1 neighbour in the North" << endl;
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
 			ppathstructNeighbour->AddNeighbour(
-				*pNewStructure, OC_DIR_S );
+				*pNewStructure, OC_DIR_O_S );
 			pNewStructure->AddNeighbour(
-				*ppathstructNeighbour, OC_DIR_N );
+				*ppathstructNeighbour, OC_DIR_O_N );
 		}
 	}
 
 // Get the neighbour in the South
-	if (gpMapMgr->GetNeighbourWH( w, h, nW, nH, OC_DIR_S ) == true) {
-		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nL, OC_DIR_O_S ) == true) {
+		pstructNeighbour = BuildingLayer::GetStructure( nW, nL );
 		if ( (pstructNeighbour != NULL)
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
 			ppathstructNeighbour->AddNeighbour(
-				*pNewStructure, OC_DIR_N );
+				*pNewStructure, OC_DIR_O_N );
 			pNewStructure->AddNeighbour(
-				*ppathstructNeighbour, OC_DIR_S );
+				*ppathstructNeighbour, OC_DIR_O_S );
 		}
 	}
 
 // Get the neighbour in the West
-	if (gpMapMgr->GetNeighbourWH( w, h, nW, nH, OC_DIR_W ) == true) {
-		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nL, OC_DIR_O_W ) == true) {
+		pstructNeighbour = BuildingLayer::GetStructure( nW, nL );
 		if ( (pstructNeighbour != NULL)
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
 			ppathstructNeighbour->AddNeighbour(
-				*pNewStructure, OC_DIR_E );
+				*pNewStructure, OC_DIR_O_E );
 			pNewStructure->AddNeighbour(
-				*ppathstructNeighbour, OC_DIR_W );
+				*ppathstructNeighbour, OC_DIR_O_W );
 		}
 	}
 
 // Get the neighbour in the East
-	if (gpMapMgr->GetNeighbourWH( w, h, nW, nH, OC_DIR_E ) == true) {
-		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nL, OC_DIR_O_E ) == true) {
+		pstructNeighbour = BuildingLayer::GetStructure( nW, nL );
 	// if the neighbour is a road structure
 	// then add this new road structure as its new neighbour
 	// we do the same with the new road structure
@@ -1007,9 +1060,9 @@ BuildingLayer::_BuildPathStructure(
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
 			ppathstructNeighbour->AddNeighbour(
-				*pNewStructure, OC_DIR_W );
+				*pNewStructure, OC_DIR_O_W );
 			pNewStructure->AddNeighbour(
-				*ppathstructNeighbour, OC_DIR_E );
+				*ppathstructNeighbour, OC_DIR_O_E );
 		}
 	}
 }
@@ -1037,7 +1090,7 @@ BuildingLayer::_BuildRCIStructure(
 //cout << "size of RCIStructure is: " << sizeof( RCIStructure ) << endl;
 
 	rCost = 0;
-	cost = gpPropertyMgr->Get( OC_BUILD_COST, enumStructCode );
+	cost = gVars.gpPropertyMgr->Get( OC_TOOL_COST, enumStructCode );
 
 // Let's GO !
 	l = L1;
@@ -1046,7 +1099,7 @@ BuildingLayer::_BuildRCIStructure(
 		w = W1;
 		while (w <= W2) {
 			if ((_tabpStructure[ linearIndex ] == NULL)
-			 && (gpMapMgr->IsSquarePlane(w, l) == true )) {
+			 && (gVars.gpMapMgr->IsSquarePlane(w, l) == true )) {
 				_tabpStructure[ linearIndex ] = new RCIStructure(enumStructCode);
 				rCost += cost;
 			}
@@ -1068,7 +1121,7 @@ BuildingLayer::_BuildFloraStructure(
 	const OPENCITY_STRUCTURE_CODE & enumStructCode,
 	uint& rCost )
 {
-	OPENCITY_DEBUG( "I'm building some flora structures" );
+//	OPENCITY_DEBUG( "I'm building some flora structures" );
 
 	static uint w, l;
 	static uint linearIndex;
@@ -1082,7 +1135,7 @@ BuildingLayer::_BuildFloraStructure(
 //cout << "size of RCIStructure is: " << sizeof( RCIStructure ) << endl;
 
 	rCost = 0;
-	cost = gpPropertyMgr->Get( OC_BUILD_COST, enumStructCode );
+	cost = gVars.gpPropertyMgr->Get( OC_TOOL_COST, enumStructCode );
 
 // Let's GO !
 	l = L1;
@@ -1091,7 +1144,7 @@ BuildingLayer::_BuildFloraStructure(
 		w = W1;
 		while (w <= W2) {
 			if ((_tabpStructure[ linearIndex ] == NULL)
-			 && (gpMapMgr->IsSquarePlane(w, l) == true )) {
+			 && (gVars.gpMapMgr->IsSquarePlane(w, l) == true )) {
 				_tabpStructure[ linearIndex ] = new TreeStructure(enumStructCode);
 				rCost += cost;
 			}
@@ -1119,7 +1172,7 @@ BuildingLayer::_BuildWEGStructure(
 	OPENCITY_GRAPHIC_CODE gcode;
 
 // Get the graphic code of the structure
-	gcode = gpPropertyMgr->GetGC( enumStructCode );
+	gcode = gVars.gpPropertyMgr->GetGC( enumStructCode );
 	if ( gcode == OC_EMPTY ) {
 		OPENCITY_DEBUG( "WARNING: not implemented" );
 		assert(0);
@@ -1127,7 +1180,7 @@ BuildingLayer::_BuildWEGStructure(
 
 // Get the corresponding WLH and calculate the range
 // NOTE: An coal electric plant is 4x4 size by default
-	gpPropertyMgr->GetWLH( gcode, sw, 4, sl, 4, sh, 1 );
+	gVars.gpPropertyMgr->GetWLH( gcode, sw, 4, sl, 4, sh, 1 );
 	W2 = W1 + sw - 1;
 	L2 = L1 + sl - 1;
 
@@ -1158,7 +1211,7 @@ BuildingLayer::_BuildWEGStructure(
 
 // Create the main structure first, but we don't insert it
 	pMainStructure = new WEGStructure( enumStructCode );
-	rCost = gpPropertyMgr->Get( OC_BUILD_COST, enumStructCode );
+	rCost = gVars.gpPropertyMgr->Get( OC_TOOL_COST, enumStructCode );
 
 // Create all the area as parts of a bigger main structure
 	l = L1;
@@ -1188,13 +1241,13 @@ BuildingLayer::_BuildWEGStructure(
    /*=====================================================================*/
 void
 BuildingLayer::_LoadStructure(
-	const uint & w1,
-	const uint & l1,
+	const uint& w1,
+	const uint& l1,
 	Structure* pMainStruct )
 {
 	OPENCITY_DEBUG( "Loading some structures" );
 
-	uint w = 0, l = 0, w2 = 0, l2 = 0;
+	uint w = 0, l = 0, w2 = 0, l2 = 0;			// Map coordinates
 	uint sw = 0, sl = 0, sh = 0;				// Structure's width, length and height
 	OPENCITY_GRAPHIC_CODE gcode = OC_EMPTY;
 	uint linearIndex = 0;
@@ -1202,13 +1255,13 @@ BuildingLayer::_LoadStructure(
 // Get the graphic code of the structure
 	gcode = pMainStruct->GetGraphicCode();
 	if ( gcode == OC_EMPTY ) {
-		OPENCITY_DEBUG( "WARNING: not implemented" );
+		OPENCITY_FATAL( "Not implemented" );
 		assert(0);
 	}
 
 // Get the corresponding WLH and calculate the range
 // NOTE: An coal electric plant is 4x4 size by default
-	gpPropertyMgr->GetWLH( gcode, sw, 1, sl, 1, sh, 1 );
+	gVars.gpPropertyMgr->GetWLH( gcode, sw, 1, sl, 1, sh, 1 );
 	w2 = w1 + sw - 1;
 	l2 = l1 + sl - 1;
 
@@ -1259,7 +1312,7 @@ BuildingLayer::_DestroyStructure(
 	if ( pstruct == NULL )
 		return OC_ERR_SOMETHING;
 
-	rCost = gpPropertyMgr->Get( OC_DESTROY_COST, pstruct->GetCode(), pstruct );
+	rCost = gVars.gpPropertyMgr->Get( OC_DESTROY_COST, pstruct->GetCode(), pstruct );
 
 // The main structure is pstruct itself
 	pstructMain = pstruct;
@@ -1280,15 +1333,16 @@ BuildingLayer::_DestroyStructure(
 		case OC_STRUCTURE_HOSPITALDEPT:
 		case OC_STRUCTURE_EDUCATIONDEPT:
 		case OC_STRUCTURE_EPLANT_COAL:
+		case OC_STRUCTURE_EPLANT_NUCLEAR:
 		// Get the width, length and height of the main structure
 		// WARNING: invalid default values given.
-			gpPropertyMgr->GetWLH( pstructMain->GetGraphicCode(), sw, 0, sl, 0, sh, 0 );
+			gVars.gpPropertyMgr->GetWLH( pstructMain->GetGraphicCode(), sw, 0, sl, 0, sh, 0 );
 			assert( (sw != 0) && (sl != 0) );
 			sw--; sl--;			// We calculate the range
 
 		// Calculate the possible area to limit cpu usage
-			gpMapMgr->GetPossibleWH( dw1, dl1, -sw, -sl );
-			gpMapMgr->GetPossibleWH( dw2, dl2,  sw,  sl );
+			gVars.gpMapMgr->GetPossibleWH( dw1, dl1, -sw, -sl );
+			gVars.gpMapMgr->GetPossibleWH( dw2, dl2,  sw,  sl );
 		// Now search for all STRUCTURE_PART and the main structure, destroy em
 			for ( tempL = dl1; tempL <= dl2; tempL++ ) {
 				linearIndex = tempL*_uiLayerWidth + dw1;
@@ -1341,43 +1395,43 @@ BuildingLayer::_DestroyPathStructure(
 
 	ppathstruct = (PathStructure*)pstruct;
 // get the neighbour in the North
-	if (gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_N ) == true) {
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_O_N ) == true) {
 		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
 	// if the neighbour is a path structure
 		if ( (pstructNeighbour != NULL)
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
-			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_S );
+			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_O_S );
 		}
 	}
 
 // get the neighbour in the South
-	if (gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_S ) == true) {
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_O_S ) == true) {
 		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
 		if ( (pstructNeighbour != NULL)
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
-			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_N );
+			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_O_N );
 		}
 	}
 
 // get the neighbour in the West
-	if (gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_W ) == true) {
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_O_W ) == true) {
 		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
 		if ( (pstructNeighbour != NULL)
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
-			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_E );
+			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_O_E );
 		}
 	}
 
 // get the neighbour in the East
-	if (gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_E ) == true) {
+	if (gVars.gpMapMgr->GetNeighbourWH( w, l, nW, nH, OC_DIR_O_E ) == true) {
 		pstructNeighbour = BuildingLayer::GetStructure( nW, nH );
 		if ( (pstructNeighbour != NULL)
 		   &&(pstructNeighbour->GetCode() == enumStructCode)) {
 			ppathstructNeighbour = (PathStructure*)pstructNeighbour;
-			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_W );
+			ppathstructNeighbour->RemoveNeighbour( *ppathstruct, OC_DIR_O_W );
 		}
 	}
 
