@@ -1,9 +1,11 @@
 /***************************************************************************
-                          gui3dview.cpp    -  description
-                             -------------------
-    begin                : lun 22 mar 2004
-    copyright            : (C) 2004 by Duong-Khang NGUYEN
-    email                : neoneurone @ users sourceforge net
+						gui3dview.cpp    -  description
+							-------------------
+	begin                : mar 22th, 2004
+	copyright            : (C) 2004 by Duong-Khang NGUYEN
+	email                : neoneurone @ users sourceforge net
+
+	$Id: gui3dview.cpp 261 2007-10-20 18:22:47Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
