@@ -1,10 +1,11 @@
 /***************************************************************************
-                          gui3dview.h    -  description
-    $Id: gui3dview.h 3 2006-06-11 08:16:14Z neoneurone $
-                             -------------------
-    begin                : lun 22 mar 2004
-    copyright            : (C) 2004 by Duong-Khang NGUYEN
-    email                : neoneurone @ users sourceforge net
+						gui3dview.h    -  description
+							-------------------
+	begin                : mar 22th, 2004
+	copyright            : (C) 2004 by Duong-Khang NGUYEN
+	email                : neoneurone @ users sourceforge net
+
+	$Id: gui3dview.h 261 2007-10-20 18:22:47Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -28,8 +29,8 @@
    */
 class GUI3DView : public GUIMain {
 public:
-	GUI3DView() {};
-	virtual ~GUI3DView() {};
+	GUI3DView() {}
+	virtual ~GUI3DView() {}
 
 };
 
