@@ -1,11 +1,11 @@
 /***************************************************************************
-                          guimain.cpp    -  description
-          $Id: guimain.cpp 3 2006-06-11 08:16:14Z neoneurone $
-                             -------------------
-    begin                : lun 22 mar 2004
-    copyright            : (C) 2004 by Duong-Khang NGUYEN
-    email                : neoneurone @ users sourceforge net
+						guimain.cpp    -  description
+							-------------------
+	begin                : march 22th, 2004
+	copyright            : (C) 2004-2006 by Duong-Khang NGUYEN
+	email                : neoneurone @ users sourceforge net
 
+	$Id: guimain.cpp 175 2007-02-22 20:14:42Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -20,6 +20,22 @@
 #include "guimain.h"
 
 
+   /*=====================================================================*/
+GUIMain::GUIMain():
+_pctr( NULL ),
+_iX( 0 ),
+_iY( 0 ),
+_uiWidth( 0 ),
+_uiHeight( 0 ),
+_ubAttribute( 0 )
+{
+}
+
+
+   /*=====================================================================*/
+GUIMain::~GUIMain()
+{
+}
 
 
 
