@@ -5,7 +5,7 @@
 	copyright            : (C) 2006 by Frdric RODRIGO
 	email                : f.rodrigo free.fr
 	
-	$Id: filter.h 42 2006-09-10 16:03:31Z neoneurone $
+	$Id: filter.h 125 2007-01-13 21:50:42Z frodrigo $
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,9 +22,11 @@
 
 #include "../map.h"			// ../ because of silly DevCpp
 
-namespace mapgen
+namespace MapGen
 {
 
+
+//========================================================================
 /** Filter abstract class/interface
 */
 class Filter {
@@ -36,15 +38,21 @@ public:
 //========================================================================
 /** Apply filter on map in place
 */
-	virtual void apply( Map* map );
+	virtual void apply( Map* map ) = 0;
+
 
 protected:
+
+//========================================================================
+/** Extract the min and the max values out of the map
+*/
 	void _getMinMax(
 		const Map* map,
-		float* min,
-		float* max );
+		float& min,
+		float& max );
 };
 
+
 }
 
 #endif
