@@ -5,7 +5,7 @@
 	copyright            : (C) 2006 by Frdric RODRIGO
 	email                : f.rodrigo free.fr
 	
-	$Id: flattern.h 20 2006-07-16 10:28:39Z neoneurone $
+	$Id: flattern.h 125 2007-01-13 21:50:42Z frodrigo $
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,14 +22,15 @@
 
 #include "filter.h"
 
-namespace mapgen
+namespace MapGen
 {
 
+
+//========================================================================
 /** Use a x^power function to resample the map
 */
 class Flattern: public Filter {
 public:
-	Flattern();
 	Flattern( const float power );
 	~Flattern();
 
@@ -40,6 +41,7 @@ private:
 
 };
 
+
 }
 
 #endif
