@@ -5,7 +5,7 @@
 	copyright            : (C) 2006 by Frdric RODRIGO
 	email                : f.rodrigo free.fr
 	
-	$Id: gaussblur.h 20 2006-07-16 10:28:39Z neoneurone $
+	$Id: gaussblur.h 124 2007-01-13 17:28:49Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,31 +22,33 @@
 
 #include "filter.h"
 
-namespace mapgen
+namespace MapGen
 {
 
+
+//========================================================================
 /** Apply a Guass blur on the map with arbitrary precision window size
 */
 class GaussBlur: public Filter {
 public:
 	GaussBlur( const uint length );
-
 	~GaussBlur();
 
 	void apply( Map* map );
 
 private:
-	uint	_length;
-	float	*_convulsion;
+	uint	_uiLength;
+	float*	_afConvulsion;
 
-	static float _gauss(
-			const uint x,
-			const uint width );
+	inline static float _gauss(
+		const uint x,
+		const uint width );
 
-	static float *_blur( const uint length );
+	static float* _blur( const uint length );
 
 };
 
+
 }
 
 #endif
