@@ -5,7 +5,7 @@
 	copyright            : (C) 2006 by Frdric RODRIGO
 	email                : f.rodrigo free.fr
 	
-	$Id: normalize.h 20 2006-07-16 10:28:39Z neoneurone $
+	$Id: normalize.h 124 2007-01-13 17:28:49Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,9 +22,11 @@
 
 #include "filter.h"
 
-namespace mapgen
+namespace MapGen
 {
 
+
+//========================================================================
 /** Linear re-range of the map
 */
 class Normalize: public Filter {
@@ -42,6 +44,7 @@ private:
 
 };
 
+
 }
 
 #endif
