@@ -5,7 +5,7 @@
 	copyright            : (C) 2006 by Frdric RODRIGO
 	email                : f.rodrigo free.fr
 	
-	$Id: generator.cpp 20 2006-07-16 10:28:39Z neoneurone $
+	$Id: generator.cpp 291 2007-11-27 21:09:32Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,13 +19,17 @@
 
 #include "generator.h"
 
-namespace mapgen
+#include <cstdlib>
+
+namespace MapGen
 {
 
+
    /*=====================================================================*/
-Generator::Generator()
+Generator::Generator( const uint seed )
 {
 	MAP_DEBUG( "ctor" );
+	srand( seed );
 }
 
 
@@ -35,11 +39,4 @@ Generator::~Generator()
 	MAP_DEBUG( "dtor" );
 }
 
-
-   /*=====================================================================*/
-Map* Generator::render()
-{
-	return new Map( 50, 50 );
-}
-
 }
