@@ -5,7 +5,7 @@
 	copyright            : (C) 2006 by Frdric RODRIGO
 	email                : f.rodrigo free.fr
 	
-	$Id: generator.h 42 2006-09-10 16:03:31Z neoneurone $
+	$Id: generator.h 125 2007-01-13 21:50:42Z frodrigo $
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,22 +22,25 @@
 
 #include "../map.h"			// ../ because of silly DevCpp
 
-namespace mapgen
+namespace MapGen
 {
 
+
+//========================================================================
 /** Generator virtual class/interface
 */
 class Generator {
 public:
-	Generator();
+	Generator( const uint seed );
 	virtual ~Generator();
 
-	virtual Map* render();
+	virtual Map* render() const = 0;
 
 private:
 
 };
 
+
 }
 
 #endif
