@@ -1,11 +1,11 @@
 /***************************************************************************
-							modelloader.cpp  -  description
-								-------------------
-	begin                : sam mai 22 2004
-	copyright            : (C) 2004 by Duong-Khang NGUYEN
+						modelloader.cpp  -  description
+							-------------------
+	begin                : may 22th, 2004
+	copyright            : (C) 2004-2007 by Duong-Khang NGUYEN
 	email                : neoneurone @ users sourceforge net
 	
-	$Id: modelloader.cpp 31 2006-08-03 20:19:52Z neoneurone $
+	$Id: modelloader.cpp 304 2007-12-22 21:08:40Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -42,9 +42,9 @@ using std::ifstream;
 using namespace AC3D;
 
 
-// Accumulate the locations command
-float locAccu[3];
-bool bNeedAlpha;
+// Local module static variables
+static float locAccu[3];		// Accumulate the locations command
+static bool bNeedAlpha;			// Alpha processing
 
 // Debug variables
 //	unsigned int nbPoly;
@@ -79,10 +79,8 @@ ModelLoader::LoadOCM(
 {
 	vector<GLfloat> vf;
 	vector<GLuint> vui;
-	GLfloat fValue;
-	OC_CHAR tempStr [OC_OCM_MAX_LINE_LENGTH];
-// TOKILL	OC_CHAR tempPath [OC_OCM_MAX_LINE_LENGTH];
-	OC_CHAR* endStr;
+	GLfloat fValue, fValue2, fValue3, fValue4;
+	char tempStr [OC_OCM_MAX_LINE_LENGTH];
 
 // model's parameter
 	GLfloat* ftab;
@@ -103,16 +101,8 @@ ModelLoader::LoadOCM(
 	while (!ocmFile.eof()) {
 //debug printf("tempstr: '%s' \n", tempStr );
 	// if not NULL nor commented out then
-		if ((strlen(tempStr) != 0)
-		&&  (tempStr[0] != '#')) {
-			fValue = strtof( tempStr, &endStr );
-
-		// is there any conversion error ?
-			if ( tempStr == endStr ) {
-				OPENCITY_DEBUG( "can not load file : " );
-				ocmFile.close();
-				return NULL;
-			}
+		if ((strlen(tempStr) != 0) and (tempStr[0] != '#')) {
+			fValue = atof( tempStr );
 
 		// WARNING no error checking below here, tired of this :)
 		// store the OCM value type code
@@ -122,48 +112,36 @@ ModelLoader::LoadOCM(
 		// read the primitive's values
 			ocmFile.getline( tempStr, OC_OCM_MAX_LINE_LENGTH );
 
-			if (fValue
-			== OC_OCM_VERTEX) {
+			if (fValue == OC_OCM_VERTEX) {
 			// read 3 floats
-				fValue = strtof( tempStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
+				sscanf( tempStr, "%f %f %f", &fValue, &fValue2, &fValue3 );
 				vf.push_back( fValue );
+				vf.push_back( fValue2 );
+				vf.push_back( fValue3 );
 			} else
-			if (fValue
-			== OC_OCM_COLOR) {
+			if (fValue == OC_OCM_COLOR) {
 			// read 4 floats
-				fValue = strtof( tempStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
+				sscanf( tempStr, "%f %f %f %f", &fValue, &fValue2, &fValue3, &fValue4 );
 				vf.push_back( fValue );
+				vf.push_back( fValue2 );
+				vf.push_back( fValue3 );
+				vf.push_back( fValue4 );
 			} else
-			if (fValue
-			== OC_OCM_TEXCOOR) {
+			if (fValue == OC_OCM_TEXCOOR) {
 			// read 3 floats
-				fValue = strtof( tempStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
-				vf.push_back( fValue );
-				fValue = strtof( endStr, &endStr );
+				sscanf( tempStr, "%f %f %f", &fValue, &fValue2, &fValue3 );
 				vf.push_back( fValue );
+				vf.push_back( fValue2 );
+				vf.push_back( fValue3 );
 			} else
-			if (fValue
-			== OC_OCM_TEXFILE) {
-				OPENCITY_DEBUG("Loading texture file");
-				vui.push_back( Texture::Load( ocHomeDirPrefix( tempStr )));
+			if (fValue == OC_OCM_TEXFILE) {
+			//	OPENCITY_DEBUG( "Loading texture file: " << ocDataDirPrefix( tempStr ) );
+				vui.push_back( Texture::Load( ocDataDirPrefix( tempStr )) );
 //debug cout << "Texture id got: " << vui.back() << endl;
 			} else
-			if (fValue
-			== OC_OCM_TEXBIND) {
+			if (fValue == OC_OCM_TEXBIND) {
 			// read 1 float
-				fValue = strtof( tempStr, &endStr );
+				fValue = atof( tempStr );
 			// WARNING: no error checking
 			// it's quite dangerous
 			// 0 is a default texture in OpenGL
@@ -202,10 +180,9 @@ Model* const
 ModelLoader::LoadAC3D(
 	const string & rcsFileName )
 {
-	AC3DModel ac3dmodel = AC3DModel( rcsFileName );
+	AC3DModel ac3dmodel( rcsFileName );
 	vector<AC3DMaterial> vMaterial;
-	map<string, GLuint> mapTexture;
-	GLuint list = 0, listAlpha = 0;
+	GLuint tex =0, list = 0, listTwoSide = 0, listAlpha = 0;
 	string strPath = "";
 
 	if (!ac3dmodel.IsGood())
@@ -244,26 +221,21 @@ ModelLoader::LoadAC3D(
 //	nbVertex = 0;
 
 // Load all the texture used by the model
-	_AC3DTextureToGL( strPath, pObject, mapTexture );
+	_AC3DTextureToGL( strPath, pObject, tex );
 
-// Debug
-	if (mapTexture.size() > 1) {
-		OPENCITY_DEBUG("WARNING: more than 1 texture used by the model");
-		assert( 0 );
-	}
 
    /*=====================================================================*/
-// Recursively load all the objects into the _opaque_ display list
+// Recursively load all the objects into the _opaque_ one side display list
 	list = glGenLists( 1 );
 	glNewList( list, GL_COMPILE );
 // Save the all enabled GL bits
 	glPushAttrib( GL_ENABLE_BIT );
-	glDisable( GL_BLEND );
+	glEnable( GL_CULL_FACE );
 // Enable the texture target and bind the _first_ texture only
-	if (mapTexture.size() > 0) {
+	if ( glIsTexture(tex) == GL_TRUE ) {
 		glEnable( GL_TEXTURE_2D );
-		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
-		glBindTexture( GL_TEXTURE_2D, (mapTexture.begin())->second);
+		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
+		glBindTexture( GL_TEXTURE_2D, tex );
 	}
 	else {
 		glDisable( GL_TEXTURE_2D );
@@ -271,13 +243,40 @@ ModelLoader::LoadAC3D(
 
 // Load all the vertex
 	glBegin( GL_TRIANGLES );
-	_AC3DVertexToGL( strPath, vMaterial, pObject, false );
+	_AC3DVertexToGL( strPath, vMaterial, pObject, false, false );
 	glEnd();
 
 // Restore all enabled bits
 	glPopAttrib();
 	glEndList();
 
+
+   /*=====================================================================*/
+// Recursively load all the objects into the _opaque_ two side display list
+	listTwoSide = glGenLists( 1 );
+	glNewList( listTwoSide, GL_COMPILE );
+// Save the all enabled GL bits
+	glPushAttrib( GL_ENABLE_BIT );
+// Enable the texture target and bind the _first_ texture only
+	if ( glIsTexture(tex) == GL_TRUE ) {
+		glEnable( GL_TEXTURE_2D );
+		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
+		glBindTexture( GL_TEXTURE_2D, tex );
+	}
+	else {
+		glDisable( GL_TEXTURE_2D );
+	}
+
+// Load all the vertex
+	glBegin( GL_TRIANGLES );
+	_AC3DVertexToGL( strPath, vMaterial, pObject, false, true );
+	glEnd();
+
+// Restore all enabled bits
+	glPopAttrib();
+	glEndList();
+
+
    /*=====================================================================*/
 	if (bNeedAlpha) {
 // Recursively load all the objects into the _alpha_ display list
@@ -289,19 +288,21 @@ ModelLoader::LoadAC3D(
 	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 	glEnable(GL_ALPHA_TEST);
 	glAlphaFunc(GL_GREATER, 0.2);
+// Test
 //	glBlendFunc( GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA );
 //	glBlendFunc( GL_ONE, GL_ZERO );
 // Enable the texture target and bind the _first_ texture only
-	if (mapTexture.size() > 0) {
+	if ( glIsTexture(tex) == GL_TRUE ) {
 		glEnable( GL_TEXTURE_2D );
 		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
+// Test
 //		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND );
 //		GLfloat env_color [] = { 0, 0, 0, 0 };
 //		GLfloat env_color [] = { 1, 1, 1, 0 };
 //		glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, env_color );
 //		GLfloat obj_color [] = { 01, 01, 01, 01 };
 //		glColor4fv( obj_color );
-		glBindTexture( GL_TEXTURE_2D, (mapTexture.begin())->second);
+		glBindTexture( GL_TEXTURE_2D, tex );
 	}
 	else {
 		glDisable( GL_TEXTURE_2D );
@@ -322,7 +323,7 @@ ModelLoader::LoadAC3D(
 //	cout << "Number of polygons: " << nbPoly
 //		 << " / vertex: " << nbVertex << endl;
 
-	return new Model( list, listAlpha, mapTexture );
+	return new Model( list, listTwoSide, listAlpha, tex );
 }
 
 
@@ -360,168 +361,55 @@ ModelLoader::GetNormal(
 
    /*=====================================================================*/
    /*                        PRIVATE      METHODS                         */
-   /*=====================================================================*/
-/* POLYGON version
-void
-ModelLoader::_AC3DToGL(
-	const string& strPath,
-	const vector<AC3DMaterial>& vMaterial,
-	const AC3DObject* const pObject,
-	map<string, GLuint>& mapTexture
-	)
-{
-	const float* loc;
-	AC3DMaterial mat;
-	vector<Vertex> vVertex;
-
-	vector<AC3DSurface*>::size_type pos, size;
-	vector<AC3DSurface*> vpSurface;
-
-	vector<AC3DObject*>::size_type posObj, sizeObj;
-	vector<AC3DObject*> vpObj;
-
-	vector<Ref>::size_type posRef, sizeRef;
-	vector<Ref> vRef;
-	Ref ref;
-
-	GLuint tex = 0;
-	bool texEnabled = false;
-
-
-	assert( pObject != NULL );
-
-// Set the texture
-	if ( pObject->GetTextureFile() != "" ) {
-		texEnabled = true;
-
-	// IF the specified texture has not been loaded already THEN
-		if (mapTexture.find( pObject->GetTextureFile() ) == mapTexture.end() ) {
-		// Load the texture as specified by the texture command
-			tex = Texture::Load( strPath + "/" + pObject->GetTextureFile() );
-			mapTexture[ pObject->GetTextureFile() ] = tex;
-		}
-		else {
-		// Assign the cached texture to the model
-			OPENCITY_DEBUG("Texture loading cache hit");
-			tex = mapTexture[ pObject->GetTextureFile() ];
-		}
-
-		glEnable( GL_TEXTURE_2D );
-		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
-		glBindTexture( GL_TEXTURE_2D, tex);
-	}
-
-	loc = pObject->GetLoc();
-	locAccu[0] += loc[0];
-	locAccu[1] += loc[1];
-	locAccu[2] += loc[2];
-
-	vVertex = pObject->GetVVertex();
-	vpSurface = pObject->GetVPSurface();
-
-	size = vpSurface.size();
-	for (pos = 0; pos < size; pos++) {
-	// Debug
-		++nbPoly;
-	// Get the material of the current surface
-		mat = vMaterial[ vpSurface[pos]->GetMat() ];
-	// Set the color of the surface with COLOR_MATERIAL enabled
-		glColor3f( mat.rgb.fR, mat.rgb.fG, mat.rgb.fB );
-
-		vRef = vpSurface[pos]->GetVRef();
-		sizeRef = vRef.size();
-	// Debug
-		cout << "Vertex per surface: " << sizeRef << endl;
-
-		glBegin( GL_POLYGON );
-		for (posRef = 0; posRef < sizeRef; posRef++) {
-		// Debug
-			++nbVertex;
-
-			ref = vRef[posRef];
-			if (texEnabled)
-				glTexCoord2f( ref.fTexS, ref.fTexT );
-			glVertex3f(
-				vVertex[ref.uiVertIndex].x + locAccu[0],
-				vVertex[ref.uiVertIndex].y + locAccu[1],
-				vVertex[ref.uiVertIndex].z + locAccu[2]
-			);
-		}
-		glEnd();
-	} // For each surface
-
-	if (texEnabled) {
-		texEnabled = false;			// Safe
-		glDisable( GL_TEXTURE_2D );
-	}
-
-// Parse all the child objects
-	vpObj = pObject->GetVPObject();
-	sizeObj = vpObj.size();
-	for (posObj = 0; posObj < sizeObj; posObj++) {
-		_AC3DToGL( strPath, vMaterial, vpObj[posObj], mapTexture );
-	}
-
-	locAccu[0] -= loc[0];
-	locAccu[1] -= loc[1];
-	locAccu[2] -= loc[2];
-}
-*/
-
-
    /*=====================================================================*/
 void
 ModelLoader::_AC3DTextureToGL
 (
 	const string& strPath,
 	const AC3DObject* const pObject,
-	map<string, GLuint>& mapTexture
+	GLuint& tex
 )
 {
 	vector<AC3DObject*>::size_type posObj, sizeObj;
 	vector<AC3DObject*> vpObj;
 
-	GLuint tex = 0;
-
 
 	assert( pObject != NULL );
 
 // Get the texture
+// WARNING: we are aware of the automanaged texture loading cache
 	if ( pObject->GetTextureFile() != "" ) {
-	// IF the specified texture has not been loaded already THEN
-		if (mapTexture.find( pObject->GetTextureFile() ) == mapTexture.end() ) {
-		// Load the texture as specified by the texture command
-			tex = Texture::Load( strPath + "/" + pObject->GetTextureFile() );
-			mapTexture[ pObject->GetTextureFile() ] = tex;
-		}
-		else {
-		// Assign the cached texture to the model
-		//	OPENCITY_DEBUG("Texture loading cache hit");
-			tex = mapTexture[ pObject->GetTextureFile() ];
-		}
+		GLuint newTex = Texture::Load( strPath + "/" + pObject->GetTextureFile() );
+		if (newTex != tex) {
+			if (tex != 0) {
+				OPENCITY_FATAL( "The model tries to use multiple textures" );
+				assert( 0 );
+			}
+			else {
+				tex = newTex;
+			}
+		} // if (newTex != tex)
 	}
 
 // Parse all the child objects and retrieve the texture
 	vpObj = pObject->GetVPObject();
 	sizeObj = vpObj.size();
 	for (posObj = 0; posObj < sizeObj; posObj++) {
-		_AC3DTextureToGL( strPath, vpObj[posObj], mapTexture );
+		_AC3DTextureToGL( strPath, vpObj[posObj], tex );
 	}
 }
 
 
    /*=====================================================================*/
-// TRIANGLES version
-//#define TRIANGULATION_TEST 1
-#undef TRIANGULATION_TEST
-#ifndef TRIANGULATION_TEST
+// TRIANGLES version, inline triangulation
 void
 ModelLoader::_AC3DVertexToGL
 (
 	const string& strPath,
 	const vector<AC3DMaterial>& vMaterial,
 	const AC3DObject* const pObject,
-	const bool bProcessTranslucent
+	const bool bProcessTranslucent,
+	const bool bProcessTwoSide
 )
 {
 	const float* loc;
@@ -541,13 +429,7 @@ ModelLoader::_AC3DVertexToGL
 // Used for normal calculation
 	Vertex v1, v2, v3, normal;
 
-
 	assert( pObject != NULL );
-
-
-// debug, location calculation
-//cout << "in  : " << locAccu[0] << "/" << locAccu[1] << "/" << locAccu[2] << endl;
-
 	loc = pObject->GetLoc();
 	locAccu[0] += loc[0];
 	locAccu[1] += loc[1];
@@ -567,6 +449,11 @@ ModelLoader::_AC3DVertexToGL
 
 	size = vpSurface.size();
 	for (pos = 0; pos < size; pos++) {
+	// Process only the polygons with the requested number of sides
+		if (!bProcessTranslucent and (bProcessTwoSide xor vpSurface[pos]->IsTwoSide())) {
+			continue;
+		}
+
 // Debug ++nbPoly;
 	// Get the material of the current surface
 		mat = vMaterial[ vpSurface[pos]->GetMat() ];
@@ -577,10 +464,7 @@ ModelLoader::_AC3DVertexToGL
 		vRef = vpSurface[pos]->GetVRef();
 		sizeRef = vRef.size();
 
-// Debug cout << "Vertex per surface: " << sizeRef << endl;
-
 		assert( sizeRef >= 3 );		// We need at least one triangle
-// Debug nbVertex += sizeRef;
 		for (posRef = 1; posRef < sizeRef-1; posRef++) {
 		// Fist vertex
 			r1 = vRef[0];
@@ -620,157 +504,16 @@ process_child_objects:
 // Parse all the child objects
 	vpObj = pObject->GetVPObject();
 	sizeObj = vpObj.size();
-//debug
-//cout << "kids: " << pObject->GetNumberKid() << " / objects: " << sizeObj << endl;
 	for (posObj = 0; posObj < sizeObj; posObj++) {
-		_AC3DVertexToGL( strPath, vMaterial, vpObj[posObj], bProcessTranslucent );
+		_AC3DVertexToGL( strPath, vMaterial, vpObj[posObj], bProcessTranslucent, bProcessTwoSide );
 	}
 
 	locAccu[0] -= loc[0];
 	locAccu[1] -= loc[1];
 	locAccu[2] -= loc[2];
-
-// debug, location calculation
-//cout << "out : " << locAccu[0] << "/" << locAccu[1] << "/" << locAccu[2] << endl;
 }
 
 
-   /*=====================================================================*/
-// TRIANGLES version
-#else
-void
-ModelLoader::_AC3DVertexToGL
-(
-	const string& strPath,
-	const vector<AC3DMaterial>& vMaterial,
-	const AC3DObject* const pObject,
-	const bool bProcessTranslucent
-)
-{
-	const float* loc;
-	AC3DMaterial mat;
-	vector<Vertex> vVertex;
-
-	vector<AC3DSurface*>::size_type pos, size;
-	vector<AC3DSurface*> vpSurface;
-
-	vector<AC3DObject*>::size_type posObj, sizeObj;
-	vector<AC3DObject*> vpObj;
-
-	vector<Ref>::size_type posRef, sizeRef;
-	vector<Ref> vRef;
-	Ref r1, r2, r3;
-
-// Used for normal calculation
-	Vertex v1, v2, v3, normal;
-
-// Triangulation
-	triangulation::Triangulation* triangulator = new triangulation::Star();
-	triangulation::Triangle* triangles = NULL;
-	triangulation::Vertex* triVertices = NULL;
-
-
-	assert( pObject != NULL );
-
-
-// debug, location calculation
-//cout << "in  : " << locAccu[0] << "/" << locAccu[1] << "/" << locAccu[2] << endl;
-
-	loc = pObject->GetLoc();
-	locAccu[0] += loc[0];
-	locAccu[1] += loc[1];
-	locAccu[2] += loc[2];
-
-// Does this object need alpha processing ?
-	if (pObject->IsTranslucent()) {
-		bNeedAlpha = true;
-	}
-
-// Process only objects that we are asked to do
-	if (bProcessTranslucent xor pObject->IsTranslucent())
-		goto process_child_objects;
-
-	vVertex = pObject->GetVVertex();
-	vpSurface = pObject->GetVPSurface();
-
-	size = vpSurface.size();
-	for (pos = 0; pos < size; pos++) {
-// Debug ++nbPoly;
-	// Get the material of the current surface
-		mat = vMaterial[ vpSurface[pos]->GetMat() ];
-	// Set the color of the surface with COLOR_MATERIAL enabled
-		glColor3f( mat.rgb.fR, mat.rgb.fG, mat.rgb.fB );
-//		glColor4f( 1, 1, 1, 1 );
-
-		vRef = vpSurface[pos]->GetVRef();
-		sizeRef = vRef.size();
-
-// Debug cout << "Vertex per surface: " << sizeRef << endl;
-
-		assert( sizeRef >= 3 );		// We need at least one triangle
-// Debug nbVertex += sizeRef;
-
-		triVertices = new triangulation::Vertex[sizeRef];
-
-		for (posRef = 0; posRef < sizeRef; posRef++) {
-			r1 = vRef[posRef];
-			triVertices[posRef].x = vVertex[r1.uiVertIndex].x + locAccu[0];
-			triVertices[posRef].y = vVertex[r1.uiVertIndex].y + locAccu[1];
-			triVertices[posRef].z = vVertex[r1.uiVertIndex].z + locAccu[2];
-			triVertices[posRef].texCoord.fS = r1.fTexS;
-			triVertices[posRef].texCoord.fT = r1.fTexT;
-		}
-
-		triangles = triangulator->process( sizeRef, triVertices );
-		delete [] triVertices;
-
-		for (posRef = 0; posRef < sizeRef-2; posRef++) {
-		// Now issue the OpenGL commands
-		//	normal = GetNormal(
-		//		triangles[posRef].a, triangles[posRef].b, triangles[posRef].c );
-		//	glNormal3f( normal.x, normal.y, normal.z );
-
-			glTexCoord2f(
-				triangles[posRef].a.texCoord.fS, triangles[posRef].a.texCoord.fT );
-			glVertex3f(
-				triangles[posRef].a.x, triangles[posRef].a.y, triangles[posRef].a.z );
-			glTexCoord2f(
-				triangles[posRef].b.texCoord.fS, triangles[posRef].b.texCoord.fT );
-			glVertex3f(
-				triangles[posRef].b.x, triangles[posRef].b.y, triangles[posRef].b.z );
-			glTexCoord2f(
-				triangles[posRef].c.texCoord.fS, triangles[posRef].c.texCoord.fT );
-			glVertex3f(
-				triangles[posRef].c.x, triangles[posRef].c.y, triangles[posRef].c.z );
-		}
-
-		delete [] triangles;
-
-	} // For each surface
-
-process_child_objects:
-
-// Parse all the child objects
-	vpObj = pObject->GetVPObject();
-	sizeObj = vpObj.size();
-//debug
-//cout << "kids: " << pObject->GetNumberKid() << " / objects: " << sizeObj << endl;
-	for (posObj = 0; posObj < sizeObj; posObj++) {
-		_AC3DVertexToGL( strPath, vMaterial, vpObj[posObj], bProcessTranslucent );
-	}
-
-	locAccu[0] -= loc[0];
-	locAccu[1] -= loc[1];
-	locAccu[2] -= loc[2];
-
-	delete triangulator;
-
-// debug, location calculation
-//cout << "out : " << locAccu[0] << "/" << locAccu[1] << "/" << locAccu[2] << endl;
-}
-#endif
-
-
 
 
 
